import logging


def get_logger(name):
    logger = logging.getLogger(name)
    logger.setLevel(logging.WARNING)
    formatter = logging.Formatter(
        "%(asctime)s - %(name)s  - %(filename)s - %(levelname)s - %(message)s"
    )
    console_handler = logging.StreamHandler()
    console_handler.setFormatter(formatter)
    logger.addHandler(console_handler)
    return logger


logger = get_logger(__name__)



import logging
import os
import secrets

from pytube import YouTube

from awdible.core.defaults import DEFAULT_VIDEO_URL, VIDEO_PREFIX


class Video:
    """Get the audio from the video"""

    DEST = "./tmp/"

    @classmethod
    def get(self, url: str) -> tuple:
        """Get the audio from the video"""

        try:
            yt = YouTube(url)
            return yt

        except Exception as e:
            logging.error(f"Error: {e}")
            logging.error(f"url {url} not correct ")
            raise AttributeError(f"Video {url} not correct")

    @classmethod
    def stream(self, yt: YouTube) -> tuple:
        try:
            media = yt.streams.filter(only_audio=True).first()
            title = yt.title.replace(" ", "_")
            return title, media

        except Exception as e:
            logging.error(f"Error: {e}")
            logging.error(f"Video {url} not found")
            raise FileNotFoundError(f"Video {url} not found")

    @classmethod
    def save(self, title: str, media: YouTube, dest: str | None = None) -> str:
        """Save the audio from the media"""

        dest = dest if dest else self.DEST

        if not os.path.exists(dest):
            os.makedirs(dest)

        out = media.download(dest)

        title += ".mp4"
        os.rename(out, dest + title)

        return dest + title


if __name__ == "__main__":
    # TEST

    url = DEFAULT_VIDEO_URL
    YouTube(url)

    # title, video = Get.audio(url)
    # out = Get.save(title, video)
    # print(out)

    url = VIDEO_PREFIX + secrets.token_hex(64)

    YouTube(url)



import os
import dotenv
from dotenv import dotenv_values


def get_config():
    """Get the config"""

    no_api_key = "No API Key Found sush as 'X-RapidAPI-Key' found in the environment variables or in .env file"
    no_host_key = "No API Host Found as 'X-RapidAPI-Host' found in the environment variables or in .env file"

    env_key = os.getenv("X-RapidAPI-Key", no_api_key)
    env_host = os.getenv("X-RapidAPI-Host", no_host_key)

    if os.path.exists(".env/.env.prod"):
        values = dotenv_values(".env/.env.prod")
    elif os.path.exists(".env"):
        values = dotenv_values(".env")

    config = {
        "X-RapidAPI-Key": values.get("X-RapidAPI-Key", env_key),
        "X-RapidAPI-Host": values.get("X-RapidAPI-Host", env_host),
    }

    return config


config = get_config()



import streamlit as st

from awdible.core.awdible import Awdible


# header image
img = "./docs/assets/img/image.png"
st.image(img)  # caption="Thanks to DALL-E for the image ;) "


# title
st.title("The Best Free Awdible App Ever Made")
st.write(
    """
Just paste the url of the video you want to download and click the button.
"""
)

# Text input
pos_url = "https://youtube.com/shorts/fcCpXScFxfM?si=xVSQZWa7MgsWOl9E"
url = st.text_input("Youtube url", pos_url)


# Button
if st.button("Submit"):
    awdible = Awdible(video=url, test_mode=False, streamlit=True)
    out = awdible.run()
    with open(out, "rb") as f:
        st.download_button("Download", f, file_name=out)  # mime="audio/mp4"



"""
Default values and constants.
"""

QUERY_PREFIX = "https://www.youtube.com/results?search_query="
VIDEO_PREFIX = "https://www.youtube.com/watch?v="

DEFAULT_VIDEO_URL = "https://www.youtube.com/watch?v=9diaThxYnKA"
DEFAULT_VIDEO_ID = "9diaThxYnKA"

DEFAULT_DEST = "./"
DEFAULT_FILE = None
DEFAULT_OUTPUT = "mp3"
DEFAULT_CROP_LIMIT = 3600
DEFAULT_CONTEXT = "fr"

DEFAULT_SEARCH = False
DEFAULT_PREFIX = False
DEFAULT_ASYNCHRONOUS = False

DEFAULT_TMP = "./tmp/"
DEFAULT_LOG = "./log/"

DEFAULT_STREAMLIT = False
DEFAULT_PORT = 8501

DEFAULT_TEST_MODE = False

DEFAULT_CONFIG = {
    "X-RapidAPI-Key": "No API Key Found sush as 'X-RapidAPI-Key' found in the environment variables or in .env file",
    "X-RapidAPI-Host": "No API Host Found as 'X-RapidAPI-Host' found in the environment variables or in .env file",
}



import ffmpeg

from awdible.logger import logger


class Crop:
    """Crop the audio"""

    BASE = "./tmp/"

    @classmethod
    def cut(
        self,
        fn: str,
        start: str,
        end: str,
        n_item: int | None,
        total_items: int | None,
    ) -> str:
        """Crop the audio"""

        # pattern
        replacer = (
            fn.replace(".mp3", "_cut.mp3")
            if not (n_item or total_items)
            else fn.replace(".mp3", f"_cut_{n_item}_{total_items}.mp3")
        )

        # update out
        out = fn.replace(".mp3", replacer)

        # split
        ffmpeg.input(fn, ss=start, to=end).output(out).run()

        return out

    @classmethod
    def split(self, fn: str, n_splits=4) -> list[str]:
        """Crop the audio and return the list of the fn"""

        # len video
        len_seconds = self.video_duration(fn)

        # compute chunks
        chunks = self._build_chunks(len_seconds, n_splits)
        n_chunks = len(chunks)

        # perfom the cuts
        fn_list = []
        for i, (start, end) in enumerate(chunks):
            fn_chunk = Crop.cut(fn, start, end, i, n_chunks)
            fn_list.append(fn_chunk)

        return fn_list

    @classmethod
    def _build_chunks(self, len_seconds: int, n_splits: int) -> list[tuple]:
        """Build the chunks"""

        chunks = []

        # compute chunks
        for i in range(n_splits):
            start = i * len_seconds // n_splits
            end = (i + 1) * len_seconds // n_splits
            chunks.append((start, end))

        # -1 sec for each start except the first
        for i, (start, end) in enumerate(chunks):
            if not i:
                continue
            start -= 1
            chunks[i] = (start, end)

        # +1 sec for the last end
        chunks[-1] = (chunks[-1][0], chunks[-1][1] + 2)

        return chunks

    @classmethod
    def video_duration(self, fn):
        """Compute the video len"""

        len_seconds = ffmpeg.probe(fn)["format"]["duration"]
        len_seconds = int(float(len_seconds)) + 1

        return len_seconds



import requests
from bs4 import BeautifulSoup

from awdible.core.defaults import QUERY_PREFIX, VIDEO_PREFIX, DEFAULT_CONFIG
from awdible.logger import logger


class Search:
    """Search for the video"""

    QUERY_PREFIX = QUERY_PREFIX
    VIDEO_PREFIX = VIDEO_PREFIX
    DEFAULT_CONFIG = DEFAULT_CONFIG

    # @classmethod
    # def find(self, keywords: str) -> list[str]:
    #     """Find the video"""

    #     keywords = keywords.replace(" ", "+")

    #     url = self.QUERY_PREFIX + keywords

    #     response = requests.get(url)
    #     response.raise_for_status()

    #     return response.text

    # @classmethod
    # def parse(self, html: str) -> list[str]:
    #     """Parse the video"""

    #     soup = BeautifulSoup(html, "html.parser")
    #     a_list = soup.find_all("a", {"id": "video-title"})

    #     logger.info(a_list[:10])
    #     href_list = [a["href"] for a in a_list]

    #     return href_list

    @classmethod
    def find(
        self,
        keywords: str,
        context: str = "fr",
        config: dict = DEFAULT_CONFIG,
    ) -> list[str]:
        """Get the list of videos"""

        context = context.lower().strip()[:2]

        params = {"q": keywords}

        if context == "fr":
            params["hl"], params["gl"] = "fr", "FR"
        else:
            params["hl"], params["gl"] = "en", "US"

        url = "https://youtube-data8.p.rapidapi.com/search/"

        response = requests.get(url, headers=config, params=params)

        return response.json()

    @classmethod
    def parse(self, json: str) -> list[str]:
        """Parse the video"""

        contents = json["contents"]

        contents = contents[:10]

        videos = [x.get("video", None) for x in contents]

        li = [(x["title"], x["videoId"]) for x in videos]

        logger.info(li)

        videos = [x["videoId"] for x in videos]

        return [self.VIDEO_PREFIX + x for x in videos]



import os

import ffmpeg

from awdible.logger import logger


class Convert:
    @classmethod
    def to_mp3(cls, src: str) -> str:
        """Convert the video to mp3"""

        logger.debug(f"Converting {src} to mp3")

        # get the destination
        dest = src.split(".")[:-1]
        dest = ".".join(dest) + ".mp3"

        # logger.debug(f"Destination: {dest}")

        # the command
        cmd = {
            "q:a": 0,
            "map": "a",
            "loglevel": "quiet",
        }

        # do the conversion
        ffmpeg.input(src).output(dest, **cmd).run()

        # del  old file
        os.remove(src)

        return dest



"""
Module for validators and decorators 
"""

import os
from awdible.logger import logger

# import logging
from abc import ABC, abstractmethod


from awdible.core.defaults import DEFAULT_CONFIG


class Validator(ABC):
    """Abstract class for validators"""

    def __set_name__(self, owner, name):
        self.private_name = "_" + name

    def __get__(self, obj, objtype=None):
        return getattr(obj, self.private_name)

    def __set__(self, obj, value):
        self.validate(value)
        setattr(obj, self.private_name, value)

    @abstractmethod
    def validate(self, value):
        pass


class Number(Validator):
    """Validator for numbers"""

    def validate(self, value):
        if not isinstance(value, (int, float)):
            raise TypeError(
                f"Expected {self.private_name[1:]} to be an int or float : received {type(value)} for {value}"
            )


class Crop(Validator):
    """Validator for numbers"""

    MIN = 0
    MAX = 3_600 * 10

    def validate(self, value):
        if not isinstance(value, (int, float, str)):
            raise TypeError(
                f"Expected {self.private_name[1:]} to be an int or float  or str: received {type(value)} for {value}"
            )

        try:
            value = int(value)
        except Exception as e:
            raise ValueError(
                f"Expected {self.private_name[1:]} to be an int : received {type(value)} for {value} : {e}"
            )

        if value < self.MIN or value > self.MAX:
            raise ValueError(
                f"Expected {self.private_name[1:]} to be between {self.MIN} and {self.MAX} : received {value}"
            )


class Bool(Validator):
    """Validator for bools"""

    def validate(self, value):
        if not isinstance(value, (bool, int)):
            raise TypeError(
                f"Expected {self.private_name[1:]} to be a bool : received {type(value)} for {value}"
            )
        try:
            bool(value)
        except Exception as e:
            raise ValueError(
                f"Impossible to cast {self.private_name[1:]} to bool received {type(value)} for {value} : {e}"
            )


class File(Validator):
    """Validator for file paths"""

    def validate(self, value):
        value = value if value else ""
        if not isinstance(value, str):
            raise TypeError(
                f"Expected {self.private_name[1:]} to be a str : received {type(value)} for {value}"
            )

        if value and (not os.path.isfile(value)):
            raise ValueError(
                f"Expected {self.private_name[1:]} to be a valid file path : received {value}"
            )


class Dir(Validator):
    """Validator for directory paths"""

    def validate(self, value):
        if not isinstance(value, str):
            raise TypeError(
                f"Expected {self.private_name[1:]} to be a str : received {type(value)} for {value}"
            )

        if not os.path.isdir(value):
            raise ValueError(
                f"Expected {self.private_name[1:]} to be a valid directory path : received {value}"
            )


class Output(Validator):
    """Validator for output file format"""

    def validate(self, value):
        if not isinstance(value, str):
            raise TypeError(
                f"Expected {self.private_name[1:]} to be a str : received {type(value)} for {value}"
            )

        if value not in ["mp3", "mp4"]:
            raise ValueError(
                f"Expected {self.private_name[1:]} to be a valid file format : received {value}"
            )


class Context(Validator):
    """Validator for context"""

    def validate(self, value):
        if not isinstance(value, str):
            raise TypeError(
                f"Expected {self.private_name[1:]} to be a str : received {type(value)} for {value}"
            )

        value = value.lower()

        if value not in ["fr", "en"]:
            raise ValueError(
                f"Expected {self.private_name[1:]} to be a valid context : received {value}"
            )


# class Config(Validator):
#     """Validator for config"""

#     DEFAULT_CONFIG = DEFAULT_CONFIG

#     def validate(self, value):
#         if not isinstance(value, dict):
#             raise TypeError(
#                 f"Expected {self.private_name[1:]} to be a dict : received {type(value)} for {value}"
#             )

#         for k, v in self.DEFAULT_CONFIG.items():
#             if k not in value.keys():
#                 logger.warning(f"Missing {k} in config. Using default value {v}")
#                 raise ValueError(
#                     f"Expected {self.private_name[1:]} to be have key {k} : {v}"
#                 )



"""
"""

import os

import requests
from bs4 import BeautifulSoup
from pytube import YouTube

from awdible.logger import logger
from awdible.config import config
from .convert import Convert
from .crop import Crop
from .defaults import (
    DEFAULT_ASYNCHRONOUS,
    DEFAULT_CONTEXT,
    DEFAULT_CROP_LIMIT,
    DEFAULT_DEST,
    DEFAULT_FILE,
    DEFAULT_OUTPUT,
    DEFAULT_PORT,
    DEFAULT_PREFIX,
    DEFAULT_SEARCH,
    DEFAULT_STREAMLIT,
    DEFAULT_TEST_MODE,
    DEFAULT_TMP,
    DEFAULT_LOG,
    DEFAULT_VIDEO_ID,
    DEFAULT_VIDEO_URL,
    VIDEO_PREFIX,
    DEFAULT_CONFIG,
)

from .validators import (
    Dir,
    File,
    Output,
    # Config,
    Context,
    Bool,
)
from .search import Search
from .video import Video

# import logging


# import logging


class Awdible:
    """The Awdible class is the core of the awdible package. It is the main class that
    is used to download the audio from a youtube video. It can be used to download

    Agrs :

    video : str | list : The video url to download from youtube. If you want to input a search query, use the --search option

    dest : str : The destination directory of the file
    file : str : The input file With list of urls/videos
    output : str : The output file name
    crop_limit : int : The limit of duration of the video to download. If the video is longer than the limit, it will be cropped into smaller parts.

    search : bool : If set, the video argument will be treated as a search query and the first result will be downloaded.
    prefix : bool : If set, can ommit the url prefix 'https://www.youtube.com/watch?v=' when passing the video url.
    asynchronous : bool : If set, the download will be asynchronous.

    streamlit : bool : If set, the download will be asynchronous.
    port : int : The port of the streamlit web app.

    test_mode : bool : If set, the download will be asynchronous.
    """

    VIDEO_PREFIX = VIDEO_PREFIX

    DEFAULT_VIDEO_URL = DEFAULT_VIDEO_URL
    DEFAULT_VIDEO_ID = DEFAULT_VIDEO_ID

    DEFAULT_DEST = DEFAULT_DEST
    DEFAULT_FILE = DEFAULT_FILE
    DEFAULT_OUTPUT = DEFAULT_OUTPUT
    DEFAULT_CROP_LIMIT = DEFAULT_CROP_LIMIT
    DEFAULT_CONTEXT = DEFAULT_CONTEXT

    DEFAULT_SEARCH = DEFAULT_SEARCH
    DEFAULT_ASYNCHRONOUS = DEFAULT_ASYNCHRONOUS
    DEFAULT_PREFIX = DEFAULT_PREFIX

    DEFAULT_TMP = DEFAULT_TMP
    DEFAULT_LOG = DEFAULT_LOG

    DEFAULT_STREAMLIT = DEFAULT_STREAMLIT
    DEFAULT_PORT = DEFAULT_PORT

    DEFAULT_TEST_MODE = DEFAULT_TEST_MODE

    DEFAULT_CONFIG = DEFAULT_CONFIG

    dest = Dir()
    file = File()
    output = Output()
    crop_limit = Crop()
    context = Context()
    search = Bool()
    prefix = Bool()
    asynchronous = Bool()
    streamlit = Bool()
    test_mode = Bool()
    # config = Config()

    def __init__(
        self,
        video: str | list = DEFAULT_VIDEO_URL,
        dest: str = DEFAULT_DEST,
        file: str = DEFAULT_FILE,
        output: str = DEFAULT_OUTPUT,
        crop_limit: int = DEFAULT_CROP_LIMIT,
        context: str = DEFAULT_CONTEXT,
        search: bool = DEFAULT_SEARCH,
        prefix: bool = DEFAULT_PREFIX,
        asynchronous: bool = DEFAULT_ASYNCHRONOUS,
        streamlit: bool = DEFAULT_STREAMLIT,
        port: int = DEFAULT_PORT,
        test_mode: bool = DEFAULT_TEST_MODE,
        config: dict = config,
    ):
        """Init the Awdible class"""
        self._awdible = None
        self.video = video
        self.video_list = video if isinstance(video, list) else [video]
        self.dest = dest
        self.file = file
        self.output = output
        self.crop_limit = crop_limit
        self.context = context

        self.search = search
        self.prefix = prefix
        self.asynchronous = asynchronous

        self.streamlit = streamlit
        self.port = port

        self.test_mode = test_mode

        self.config = config

    def run(self):
        """Run the awdible session"""

        # check default tmp do exist

        if self.test_mode or self.streamlit:
            if not os.path.exists(self.DEFAULT_TMP):
                os.makedirs(self.DEFAULT_TMP)

        if not os.path.exists(self.DEFAULT_LOG):
            os.makedirs(self.DEFAULT_LOG)

        self.dest = self.DEFAULT_TMP
        self.log = self.DEFAULT_LOG

        if self.asynchronous:
            outs = self.run_asynch()
        else:
            outs = self.run_synch()

    def run_synch(self):

        outs = [self.run_one(video) for video in self.video_list]

        if len(outs) == 1:
            return outs[0]

        return outs

    def run_asynch(self):

        raise NotImplementedError("Sorry Bro! ")

    def run_one(self, video: str):
        """Run the awdible session for one video"""

        out = ""

        # good case
        if video.startswith(self.VIDEO_PREFIX):
            out = self._get_stream_save_convert(video)

        # if not valid and search is True:
        elif (not video.startswith(self.VIDEO_PREFIX)) and self.search:
            url = self._find_parse(video)[0]
            out = self._get_stream_save_convert(url)

        # if not valid and search is True:
        elif (not video.startswith(self.VIDEO_PREFIX)) and self.prefix:
            video = self.VIDEO_PREFIX + video
            out = self._get_stream_save_convert(video)

        # try  adding the prefix
        else:  # and self.default_prefix
            video = self.VIDEO_PREFIX + video

            try:
                out = self._get_stream_save_convert(video)
            except Exception as e:
                logger.error(f"Error: {e}")

            try:
                url = self._find_parse(video)[0]
                out = self._get_stream_save_convert(url)
            except Exception as e:
                logger.error(f"Error: {e}")

        logger.warning(f"Downloaded  OK: {out}")

        return out

    def _get(self, url: str) -> tuple:
        """Get the audio from the video"""

        return Video.get(url)

    def _stream(self, yt: YouTube) -> tuple:
        """Get the audio from the video"""

        return Video.stream(yt)

    def _save(self, title: str, media: YouTube) -> str:
        """Save the audio from the video"""

        return Video.save(title, media, self.dest)

    def _get_stream(self, url: str) -> tuple:
        """Get the audio from the video"""

        yt = self._get(url)
        title, media = self._stream(yt)
        return title, media

    def _get_stream_save(self, url):
        """Get the audio from the video and save it"""

        title, media = self._get_stream(url)
        out = self._save(title, media)
        return out

    def _get_stream_save_convert(self, url):
        """Get the audio from the video and save it"""

        src = self._get_stream_save(url)
        out = self._convert_to_mp3(src)
        return out

    def _find(self, keywords: str) -> list[str]:
        """Find the video"""

        json = Search.find(keywords, context=self.context, config=self.config)
        return json

    def _parse(self, json: str) -> list[str]:
        """Parse the video"""

        href_list = Search.parse(json)
        return href_list

    def _find_parse(self, keywords: str) -> list[str]:
        """Find and parse the video"""

        json = self._find(keywords)
        href_list = self._parse(json)
        return href_list

    def _convert_to_mp3(self, src: str) -> str:
        """Convert the video to mp3"""

        dest = Convert.to_mp3(src)
        return dest
